@@ -2,8 +2,8 @@
 
 use crate::engine::context::RequestContext;
 use crate::engine::database::{
-    env_string, env_u16, Comparison, DatabaseClient, DatabaseEndpoint, DatabasePool, NodeQueryVar,
-    Operation, QueryFragment, RelQueryVar, SuffixGenerator, Transaction,
+    env_string, env_u16, Comparison, DatabaseEndpoint, DatabasePool, NodeQueryVar, Operation,
+    QueryFragment, QueryResult, RelQueryVar, SuffixGenerator, Transaction,
 };
 use crate::engine::objects::{Node, NodeRef, Rel};
 use crate::engine::schema::Info;
@@ -20,6 +20,7 @@ use mobc_boltrs::BoltConnectionManager;
 use std::collections::HashMap;
 use std::convert::{TryFrom, TryInto};
 use std::iter::FromIterator;
+use uuid::Uuid;
 
 /// A Neo4J endpoint collects the information necessary to generate a connection string and
 /// build a database connection pool.
@@ -33,18 +34,22 @@ use std::iter::FromIterator;
 /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
 ///     let ne = Neo4jEndpoint::new(
 ///         "127.0.0.1".to_string(),
+///         Some("127.0.0.1".to_string()),
 ///         7687,
 ///         "neo4j".to_string(),
-///         "password".to_string()
+///         "password".to_string(),
+///         8
 ///     );
 /// #    Ok(())
 /// # }
 /// ```
 pub struct Neo4jEndpoint {
     host: String,
+    read_host: String,
     port: u16,
     user: String,
     pass: String,
+    pool_size: u16,
 }
 
 impl Neo4jEndpoint {
@@ -59,28 +64,41 @@ impl Neo4jEndpoint {
     /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
     ///     let ne = Neo4jEndpoint::new(
     ///         "127.0.0.1".to_string(),
+    ///         Some("127.0.0.1".to_string()),
     ///         7687,
     ///         "neo4j".to_string(),
-    ///         "password".to_string()
+    ///         "password".to_string(),
+    ///         8
     ///     );
     /// #    Ok(())
     /// # }
     /// ```
-    pub fn new(host: String, port: u16, user: String, pass: String) -> Self {
+    pub fn new(
+        host: String,
+        read_host_opt: Option<String>,
+        port: u16,
+        user: String,
+        pass: String,
+        pool_size: u16,
+    ) -> Self {
         Neo4jEndpoint {
-            host,
+            host: host.to_string(),
+            read_host: read_host_opt.unwrap_or(host),
             port,
             user,
             pass,
+            pool_size,
         }
     }
 
     /// Reads an variable to construct a [`Neo4jEndpoint`]. The environment variable is
     ///
     /// * WG_NEO4J_ADDR - the address for the Neo4J DB. For example, `127.0.0.1`.
+    /// * WG_NEO4J_READ_REPLICAS - the address for Neo4J read replicas. For example `127.0.0.1`. Optional.
     /// * WG_NEO4J_PORT - the port number for the Neo4J DB.  For example, `7687`.
     /// * WG_NEO4J_USER - the username for the Neo4J DB. For example, `neo4j`.
     /// * WG_NEO4J_PASS - the password for the Neo4J DB. For example, `my-db-pass`.
+    /// * WG_POOL_SIZE - connection pool size. For example, `4`. Optional.
     ///
     /// [`Neo4jEndpoint`]: ./struct.Neo4jEndpoint.html
     ///
@@ -103,9 +121,13 @@ impl Neo4jEndpoint {
     pub fn from_env() -> Result<Self, Error> {
         Ok(Neo4jEndpoint {
             host: env_string("WG_NEO4J_HOST")?,
+            read_host: env_string("WG_NEO4J_READ_REPLICAS")
+                .or_else(|_| env_string("WG_NEO4J_HOST"))?,
             port: env_u16("WG_NEO4J_PORT")?,
             user: env_string("WG_NEO4J_USER")?,
             pass: env_string("WG_NEO4J_PASS")?,
+            pool_size: env_u16("WG_POOL_SIZE")
+                .unwrap_or_else(|_| num_cpus::get().try_into().unwrap_or(8)),
         })
     }
 }
@@ -115,7 +137,7 @@ impl DatabaseEndpoint for Neo4jEndpoint {
     type PoolType = Neo4jDatabasePool;
 
     async fn pool(&self) -> Result<Self::PoolType, Error> {
-        let manager = BoltConnectionManager::new(
+        let rw_manager = BoltConnectionManager::new(
             self.host.to_string() + ":" + &*self.port.to_string(),
             None,
             [4, 0, 0, 0],
@@ -128,10 +150,26 @@ impl DatabaseEndpoint for Neo4jEndpoint {
         )
         .await?;
 
+        let ro_manager = BoltConnectionManager::new(
+            self.read_host.to_string() + ":" + &*self.port.to_string(),
+            None,
+            [4, 0, 0, 0],
+            HashMap::from_iter(vec![
+                ("user_agent", "warpgrapher/0.2.0"),
+                ("scheme", "basic"),
+                ("principal", &self.user),
+                ("credentials", &self.pass),
+            ]),
+        )
+        .await?;
+
         let pool = Neo4jDatabasePool::new(
             Pool::builder()
-                .max_open(num_cpus::get().try_into().unwrap_or(8))
-                .build(manager),
+                .max_open(self.pool_size.into())
+                .build(rw_manager),
+            Pool::builder()
+                .max_open(self.pool_size.into())
+                .build(ro_manager),
         );
 
         Ok(pool)
@@ -140,12 +178,13 @@ impl DatabaseEndpoint for Neo4jEndpoint {
 
 #[derive(Clone)]
 pub struct Neo4jDatabasePool {
-    pool: Pool<BoltConnectionManager>,
+    rw_pool: Pool<BoltConnectionManager>,
+    ro_pool: Pool<BoltConnectionManager>,
 }
 
 impl Neo4jDatabasePool {
-    fn new(pool: Pool<BoltConnectionManager>) -> Self {
-        Neo4jDatabasePool { pool }
+    fn new(rw_pool: Pool<BoltConnectionManager>, ro_pool: Pool<BoltConnectionManager>) -> Self {
+        Neo4jDatabasePool { rw_pool, ro_pool }
     }
 }
 
@@ -153,12 +192,12 @@ impl Neo4jDatabasePool {
 impl DatabasePool for Neo4jDatabasePool {
     type TransactionType = Neo4jTransaction;
 
-    async fn transaction(&self) -> Result<Self::TransactionType, Error> {
-        Ok(Neo4jTransaction::new(self.pool.get().await?))
+    async fn read_transaction(&self) -> Result<Self::TransactionType, Error> {
+        Ok(Neo4jTransaction::new(self.ro_pool.get().await?))
     }
 
-    async fn client(&self) -> Result<DatabaseClient, Error> {
-        Ok(DatabaseClient::Neo4j(Box::new(self.pool.get().await?)))
+    async fn transaction(&self) -> Result<Self::TransactionType, Error> {
+        Ok(Neo4jTransaction::new(self.rw_pool.get().await?))
     }
 }
 
@@ -316,11 +355,39 @@ impl Transaction for Neo4jTransaction {
         }
     }
 
-    #[tracing::instrument(name="wg-neo4j-create-node", skip(self, node_var, props, _partition_key_opt, info, _sg))]
+    #[tracing::instrument(name = "wg-neo4j-execute-query", skip(self, query, params))]
+    async fn execute_query<RequestCtx: RequestContext>(
+        &mut self,
+        query: String,
+        params: HashMap<String, Value>,
+    ) -> Result<QueryResult, Error> {
+        trace!(
+            "Neo4jTransaction::execute_query called -- query: {}, params: {:#?}",
+            query,
+            params
+        );
+
+        let p = Params::from(params);
+        self.client.run_with_metadata(query, Some(p), None).await?;
+
+        let pull_meta = Metadata::from_iter(vec![("n", -1)]);
+        let (response, records) = self.client.pull(Some(pull_meta)).await?;
+        match response {
+            Message::Success(_) => (),
+            message => return Err(Error::Neo4jQueryFailed { message }),
+        }
+
+        Ok(QueryResult::Neo4j(records))
+    }
+
+    #[tracing::instrument(
+        name = "wg-neo4j-create-node",
+        skip(self, node_var, props, _partition_key_opt, info, _sg)
+    )]
     async fn create_node<RequestCtx: RequestContext>(
         &mut self,
         node_var: &NodeQueryVar,
-        props: HashMap<String, Value>,
+        mut props: HashMap<String, Value>,
         _partition_key_opt: Option<&Value>,
         info: &Info,
         _sg: &mut SuffixGenerator,
@@ -331,9 +398,16 @@ impl Transaction for Neo4jTransaction {
             props
         );
 
+        if !props.contains_key("id") {
+            props.insert(
+                "id".to_string(),
+                Value::String(Uuid::new_v4().to_hyphenated().to_string()),
+            );
+        }
+
         let query = "CREATE (n:".to_string()
             + node_var.label()?
-            + " { id: randomUUID() })\n"
+            + ")\n"
             + "SET n += $props\n"
             + "RETURN n\n";
 
@@ -362,13 +436,26 @@ impl Transaction for Neo4jTransaction {
             .ok_or(Error::ResponseSetNotFound)
     }
 
-    #[tracing::instrument(name="wg-neo4j-create-rels", skip(self, src_fragment, dst_fragment, rel_var, props, props_type_name, partition_key_opt, _sg))]
+    #[tracing::instrument(
+        name = "wg-neo4j-create-rels",
+        skip(
+            self,
+            src_fragment,
+            dst_fragment,
+            rel_var,
+            props,
+            props_type_name,
+            partition_key_opt,
+            _sg
+        )
+    )]
     async fn create_rels<RequestCtx: RequestContext>(
         &mut self,
         src_fragment: QueryFragment,
         dst_fragment: QueryFragment,
         rel_var: &RelQueryVar,
-        props: HashMap<String, Value>,
+        id_opt: Option<Value>,
+        mut props: HashMap<String, Value>,
         props_type_name: Option<&str>,
         partition_key_opt: Option<&Value>,
         _sg: &mut SuffixGenerator,
@@ -396,7 +483,11 @@ impl Transaction for Neo4jTransaction {
             + rel_var.name()
             + ":"
             + rel_var.label()
-            + " { id: randomUUID() }]->("
+            + if id_opt.is_none() {
+                " { id: randomUUID() }]->("
+            } else {
+                "]->("
+            }
             + rel_var.dst().name()
             + ")\n"
             + "SET "
@@ -410,6 +501,10 @@ impl Transaction for Neo4jTransaction {
             rel_var.dst().name(),
         );
 
+        if let Some(id_val) = id_opt {
+            props.insert("id".to_string(), id_val);
+        }
+
         let mut params = src_fragment.params();
         params.extend(dst_fragment.params());
         params.insert("props".to_string(), props.into());
@@ -527,7 +622,10 @@ impl Transaction for Neo4jTransaction {
         Ok(qf)
     }
 
-    #[tracing::instrument(name="wg-neo4j-read-nodes", skip(self, query_fragment, node_var, _partition_key_opt, info))]
+    #[tracing::instrument(
+        name = "wg-neo4j-read-nodes",
+        skip(self, query_fragment, node_var, _partition_key_opt, info)
+    )]
     async fn read_nodes<RequestCtx: RequestContext>(
         &mut self,
         node_var: &NodeQueryVar,
@@ -689,7 +787,10 @@ impl Transaction for Neo4jTransaction {
         Ok(qf)
     }
 
-    #[tracing::instrument(name="wg-neo4j-read-rels", skip(self, query_fragment, rel_var, props_type_name, partition_key_opt))]
+    #[tracing::instrument(
+        name = "wg-neo4j-read-rels",
+        skip(self, query_fragment, rel_var, props_type_name, partition_key_opt)
+    )]
     async fn read_rels<RequestCtx: RequestContext>(
         &mut self,
         query_fragment: QueryFragment,
@@ -735,7 +836,10 @@ impl Transaction for Neo4jTransaction {
         Neo4jTransaction::rels(records, partition_key_opt, props_type_name)
     }
 
-    #[tracing::instrument(name="wg-neo4j-update-nodes", skip(self, query_fragment, node_var, props, _partition_key_opt, info, _sg))]
+    #[tracing::instrument(
+        name = "wg-neo4j-update-nodes",
+        skip(self, query_fragment, node_var, props, _partition_key_opt, info, _sg)
+    )]
     async fn update_nodes<RequestCtx: RequestContext>(
         &mut self,
         query_fragment: QueryFragment,
@@ -790,7 +894,18 @@ impl Transaction for Neo4jTransaction {
         Neo4jTransaction::nodes(records, info)
     }
 
-    #[tracing::instrument(name="wg-neo4j-update-rels", skip(self, query_fragment, rel_var, props, props_type_name, partition_key_opt, _sg))]
+    #[tracing::instrument(
+        name = "wg-neo4j-update-rels",
+        skip(
+            self,
+            query_fragment,
+            rel_var,
+            props,
+            props_type_name,
+            partition_key_opt,
+            _sg
+        )
+    )]
     async fn update_rels<RequestCtx: RequestContext>(
         &mut self,
         query_fragment: QueryFragment,
@@ -845,7 +960,10 @@ impl Transaction for Neo4jTransaction {
         Neo4jTransaction::rels(records, partition_key_opt, props_type_name)
     }
 
-    #[tracing::instrument(name="wg-neo4j-delete-nodes", skip(self, query_fragment, node_var, _partition_key_opt))]
+    #[tracing::instrument(
+        name = "wg-neo4j-delete-nodes",
+        skip(self, query_fragment, node_var, _partition_key_opt)
+    )]
     async fn delete_nodes(
         &mut self,
         query_fragment: QueryFragment,
@@ -893,7 +1011,10 @@ impl Transaction for Neo4jTransaction {
         Neo4jTransaction::extract_count(records)
     }
 
-    #[tracing::instrument(name="wg-neo4j-delete-rels", skip(self, query_fragment, rel_var, _partition_key_opt))]
+    #[tracing::instrument(
+        name = "wg-neo4j-delete-rels",
+        skip(self, query_fragment, rel_var, _partition_key_opt)
+    )]
     async fn delete_rels(
         &mut self,
         query_fragment: QueryFragment,
@@ -941,13 +1062,13 @@ impl Transaction for Neo4jTransaction {
         Neo4jTransaction::extract_count(records)
     }
 
-    #[tracing::instrument(name="wg-neo4j-commit-tx", skip(self))]
+    #[tracing::instrument(name = "wg-neo4j-commit-tx", skip(self))]
     async fn commit(&mut self) -> Result<(), Error> {
         debug!("transaction::commit called");
         Ok(self.client.commit().await.map(|_| ())?)
     }
 
-    #[tracing::instrument(name="wg-neo4j-rollback-tx", skip(self))]
+    #[tracing::instrument(name = "wg-neo4j-rollback-tx", skip(self))]
     async fn rollback(&mut self) -> Result<(), Error> {
         debug!("transaction::rollback called");
         Ok(self.client.rollback().await.map(|_| ())?)
@@ -1017,7 +1138,7 @@ impl<RequestCtx: RequestContext> TryFrom<bolt_proto::value::Value> for Node<Requ
         match value {
             bolt_proto::value::Value::Node(n) => {
                 let type_name = &n.labels()[0];
-                let properties: &HashMap<String, bolt_proto::value::Value> = &n.properties();
+                let properties: &HashMap<String, bolt_proto::value::Value> = n.properties();
                 let props_value = Value::try_from(properties.clone())?;
                 let props = HashMap::<String, Value>::try_from(props_value)?;
                 Ok(Node::new(type_name.to_string(), props))
