@@ -28,14 +28,14 @@ use std::convert::TryInto;
 ///
 /// ```rust
 /// # use warpgrapher::Error;
-/// # use warpgrapher::engine::config::{Configuration, Property};
+/// # use warpgrapher::engine::config::{Configuration, Property, UsesFilter};
 ///
 /// fn before_engine_build_func(config: &mut Configuration) -> Result<(), Error> {
 ///     for t in config.model.iter_mut() {
 ///         let mut_props: &mut Vec<Property> = t.mut_props();
 ///         mut_props.push(Property::new(
 ///             "global_prop".to_string(),
-///             false,
+///             UsesFilter::all(),
 ///             "String".to_string(),
 ///             false,
 ///             false,
@@ -1163,10 +1163,7 @@ where
                 src: "".to_string(),
                 dst: "".to_string(),
             })?),
-            &Info::new(
-                format!("{}QueryInput", type_name.to_string()),
-                info.type_defs(),
-            ),
+            &Info::new(format!("{}QueryInput", type_name), info.type_defs()),
             partition_key_opt,
             &mut sg,
             self.transaction,
@@ -1220,7 +1217,10 @@ where
                 src: "".to_string(),
                 dst: "".to_string(),
             })?,
-            &Info::new(type_name.to_string(), self.info.type_defs()),
+            &Info::new(
+                format!("{}CreateMutationInput", type_name),
+                self.info.type_defs(),
+            ),
             partition_key_opt,
             &mut sg,
             self.transaction,
@@ -1277,10 +1277,7 @@ where
                 src: "".to_string(),
                 dst: "".to_string(),
             })?,
-            &Info::new(
-                format!("{}UpdateInput", type_name.to_string()),
-                self.info.type_defs(),
-            ),
+            &Info::new(format!("{}UpdateInput", type_name), self.info.type_defs()),
             partition_key_opt,
             &mut sg,
             self.transaction,
@@ -1336,10 +1333,7 @@ where
                 src: "".to_string(),
                 dst: "".to_string(),
             })?,
-            &Info::new(
-                format!("{}DeleteInput", type_name.to_string()),
-                self.info.type_defs(),
-            ),
+            &Info::new(format!("{}DeleteInput", type_name), self.info.type_defs()),
             partition_key_opt,
             &mut sg,
             self.transaction,
@@ -1419,7 +1413,7 @@ where
             &info,
             partition_key_opt,
             &mut sg,
-            &mut self.transaction,
+            self.transaction,
         )
         .await?;
 
