@@ -3,26 +3,19 @@
 
 use crate::engine::context::GraphQLContext;
 use crate::engine::context::RequestContext;
-#[cfg(any(feature = "cosmos", feature = "gremlin", feature = "neo4j"))]
-use crate::engine::database::DatabaseClient;
 use crate::engine::database::{
-    DatabaseEndpoint, DatabasePool, NodeQueryVar, SuffixGenerator, Transaction,
+    DatabaseEndpoint, DatabasePool, NodeQueryVar, RelQueryVar, SuffixGenerator, Transaction,
 };
 use crate::engine::objects::resolvers::visitors::{
     visit_node_create_mutation_input, visit_node_query_input, visit_node_update_input,
+    visit_rel_query_input,
 };
 use crate::engine::objects::{Node, NodeRef, Rel};
 use crate::engine::schema::Info;
 use crate::engine::value::Value;
 use crate::juniper::BoxFuture;
 use crate::Error;
-#[cfg(any(feature = "cosmos", feature = "gremlin"))]
-use gremlin_client::GremlinClient;
 use inflector::Inflector;
-#[cfg(feature = "neo4j")]
-use mobc::Connection;
-#[cfg(feature = "neo4j")]
-use mobc_boltrs::BoltConnectionManager;
 use std::collections::HashMap;
 use std::convert::{TryFrom, TryInto};
 
@@ -68,8 +61,6 @@ where
     parent: Object<'a, RequestCtx>,
     partition_key_opt: Option<&'a Value>,
     executor: &'a Executor<'a, 'a, GraphQLContext<RequestCtx>>,
-    transaction: &'a mut <<<RequestCtx as RequestContext>::DBEndpointType as DatabaseEndpoint>::PoolType as DatabasePool>::TransactionType,
-
 }
 
 impl<'a, RequestCtx> ResolverFacade<'a, RequestCtx>
@@ -83,7 +74,6 @@ where
         parent: Object<'a, RequestCtx>,
         partition_key_opt: Option<&'a Value>,
         executor: &'a Executor<GraphQLContext<RequestCtx>>,
-        transaction: &'a mut <<<RequestCtx as RequestContext>::DBEndpointType as DatabaseEndpoint>::PoolType as DatabasePool>::TransactionType,
     ) -> Self {
         ResolverFacade {
             field_name,
@@ -92,7 +82,6 @@ where
             parent,
             partition_key_opt,
             executor,
-            transaction,
         }
     }
 
@@ -148,133 +137,6 @@ where
         self.executor.context().metadata()
     }
 
-    /// Returns a neo4j database driver from the pool.
-    ///
-    /// # Errors
-    ///
-    /// Returns an [`Error]` variant [`DatabaseNotFound`] if a neo4j database pool
-    /// is not found
-    ///
-    /// [`Error`]: ../../error/enum.Error.html
-    /// [`DatabaseNotFound`]: ../../error/enum.Error.html#variant.DatabaseNotFound
-    ///
-    /// # Examples
-    ///
-    /// ```rust, no_run
-    /// # use warpgrapher::engine::context::RequestContext;
-    /// # use warpgrapher::engine::database::neo4j::Neo4jEndpoint;
-    /// # use warpgrapher::engine::resolvers::{ResolverFacade, ExecutionResult};
-    ///
-    /// # #[derive(Clone, Debug)]
-    /// # struct AppCtx {}
-    ///
-    /// # impl RequestContext for AppCtx {
-    /// #    type DBEndpointType = Neo4jEndpoint;
-    /// #    fn new() -> Self {AppCtx {}}
-    /// # }
-    ///
-    /// async fn custom_resolve(facade: ResolverFacade<'_, AppCtx>) -> ExecutionResult {
-    ///     let neo4j_client = facade.db_into_neo4j().await?;
-    ///     // use client
-    ///
-    ///     facade.resolve_null()
-    /// }
-    /// ```
-    #[cfg(feature = "neo4j")]
-    pub async fn db_into_neo4j(&self) -> Result<Box<Connection<BoltConnectionManager>>, Error> {
-        if let DatabaseClient::Neo4j(client) = self.executor().context().pool().client().await? {
-            Ok(client)
-        } else {
-            Err(Error::DatabaseNotFound)
-        }
-    }
-
-    /// Returns a cosmos database client from the pool
-    ///
-    /// # Errors
-    ///
-    /// Returns an [`Error]` variant [`DatabaseNotFound`] if a neo4j database pool
-    /// is not found
-    ///
-    /// [`Error`]: ../../error/enum.Error.html
-    /// [`DatabaseNotFound`]: ../../error/enum.Error.html#variant.DatabaseNotFound
-    ///
-    /// # Examples
-    ///
-    /// ```rust, no_run
-    /// # use warpgrapher::engine::context::RequestContext;
-    /// # use warpgrapher::engine::database::gremlin::CosmosEndpoint;
-    /// # use warpgrapher::engine::resolvers::{ResolverFacade, ExecutionResult};
-    ///
-    /// # #[derive(Clone, Debug)]
-    /// # struct AppCtx {}
-    ///
-    /// # impl RequestContext for AppCtx {
-    /// #    type DBEndpointType = CosmosEndpoint;
-    /// #    fn new() -> Self {AppCtx {}}
-    /// # }
-    ///
-    /// async fn custom_resolve(facade: ResolverFacade<'_, AppCtx>) -> ExecutionResult {
-    ///
-    ///     let cosmos_client = facade.db_into_cosmos().await?;
-    ///     
-    ///     // use client
-    ///
-    ///     facade.resolve_null()
-    /// }
-    /// ```
-    #[cfg(feature = "cosmos")]
-    pub async fn db_into_cosmos(&self) -> Result<Box<GremlinClient>, Error> {
-        if let DatabaseClient::Gremlin(client) = self.executor().context().pool().client().await? {
-            Ok(client)
-        } else {
-            Err(Error::DatabaseNotFound)
-        }
-    }
-
-    /// Returns a gremlin database client from the pool
-    ///
-    /// # Errors
-    ///
-    /// Returns an [`Error]` variant [`DatabaseNotFound`] if a neo4j database pool
-    /// is not found
-    ///
-    /// [`Error`]: ../../error/enum.Error.html
-    /// [`DatabaseNotFound`]: ../../error/enum.Error.html#variant.DatabaseNotFound
-    ///
-    /// # Examples
-    ///
-    /// ```rust, no_run
-    /// # use warpgrapher::engine::context::RequestContext;
-    /// # use warpgrapher::engine::database::gremlin::GremlinEndpoint;
-    /// # use warpgrapher::engine::resolvers::{ResolverFacade, ExecutionResult};
-    ///
-    /// # #[derive(Clone, Debug)]
-    /// # struct AppCtx {}
-    ///
-    /// # impl RequestContext for AppCtx {
-    /// #    type DBEndpointType = GremlinEndpoint;
-    /// #    fn new() -> Self {AppCtx {}}
-    /// # }
-    ///
-    /// async fn custom_resolve(facade: ResolverFacade<'_, AppCtx>) -> ExecutionResult {
-    ///
-    ///     let gremlin_client = facade.db_into_gremlin().await?;
-    ///     
-    ///     // use client
-    ///
-    ///     facade.resolve_null()
-    /// }
-    /// ```
-    #[cfg(feature = "gremlin")]
-    pub async fn db_into_gremlin(&self) -> Result<Box<GremlinClient>, Error> {
-        if let DatabaseClient::Gremlin(client) = self.executor().context().pool().client().await? {
-            Ok(client)
-        } else {
-            Err(Error::DatabaseNotFound)
-        }
-    }
-
     /// Returns the arguments provided to the resolver in the GraphQL query
     ///
     /// # Examples
@@ -338,26 +200,35 @@ where
     /// ```rust, no_run
     /// # use serde_json::json;
     /// # use warpgrapher::Error;
+    /// # use warpgrapher::engine::database::{DatabasePool, Transaction};
     /// # use warpgrapher::engine::resolvers::{ResolverFacade, ExecutionResult};
     /// # use warpgrapher::engine::value::Value;
     /// # use warpgrapher::juniper::BoxFuture;
     ///
     /// fn custom_resolve(mut facade: ResolverFacade<()>) -> BoxFuture<ExecutionResult> {
     ///     Box::pin(async move {
-    ///         let result = facade.read_nodes("User", json!({"name": "alice"}), None)
-    ///             .await?;
+    ///         let mut transaction = facade.executor().context().pool().transaction().await?;
+    ///         transaction.begin().await?;
+    ///
+    ///         let result = ResolverFacade::read_nodes(&facade, "User", json!({"name": "alice"}),
+    ///             None, &mut transaction).await?;
     ///         let alice = result.first().unwrap();
+    ///
+    ///         transaction.commit().await?;
+    ///         std::mem::drop(transaction);
+    ///
     ///         facade.resolve_node(&alice).await
     ///     })
     /// }
     /// ```
     pub async fn read_nodes(
-        &mut self,
+        rf: &ResolverFacade<'_, RequestCtx>,
         type_name: &str,
         input: impl TryInto<Value>,
         partition_key_opt: Option<&Value>,
+        transaction: &'a mut <<<RequestCtx as RequestContext>::DBEndpointType as DatabaseEndpoint>::PoolType as DatabasePool>::TransactionType,
     ) -> Result<Vec<Node<RequestCtx>>, Error> {
-        let mut info = self.info.clone();
+        let mut info = rf.info.clone();
         info.name = "Query".to_string();
         let mut sg = SuffixGenerator::new();
         let node_var =
@@ -368,22 +239,100 @@ where
                 src: "".to_string(),
                 dst: "".to_string(),
             })?),
-            &Info::new(
-                format!("{}QueryInput", type_name.to_string()),
-                info.type_defs(),
-            ),
+            &Info::new(format!("{}QueryInput", type_name), info.type_defs()),
             partition_key_opt,
             &mut sg,
-            self.transaction,
+            transaction,
         )
         .await?;
-        let results = self
-            .transaction
+        let results = transaction
             .read_nodes(&node_var, query_fragment, partition_key_opt, &info)
             .await;
         results
     }
 
+    /// Provides an abstracted database rel read operation using warpgrapher inputs. This is the
+    /// recommended way to read relationships in a database-agnostic way that ensures the event handlers
+    /// are portable across different databases.
+    ///
+    /// # Arguments
+    ///
+    /// * `src_node_label` - String reference represing name of node type (ex: "User").
+    /// * `rel_label` - String reference representing the name of the relationship (ex: "teams").
+    /// * `input` - `Value` describing the relationship query input.
+    /// * `partition_key_opt` - Optional `Value` describing the partition key if the underlying database supports it.
+    ///
+    /// # Examples
+    ///
+    /// ```rust, no_run
+    /// # use serde_json::json;
+    /// # use warpgrapher::Error;
+    /// # use warpgrapher::engine::events::EventFacade;
+    /// # use warpgrapher::engine::objects::Rel;
+    /// # use warpgrapher::engine::value::Value;
+    /// # use warpgrapher::juniper::BoxFuture;
+    ///
+    /// fn before_user_read(value: Value, mut ef: EventFacade<()>) -> BoxFuture<Result<Value, Error>> {
+    ///     Box::pin(async move {
+    ///         let rels: Vec<Rel<()>> = ef.read_rels(
+    ///             "User",
+    ///             "teams",
+    ///             json!({
+    ///                 "src": {"name": {"EQ": "alice"}}
+    ///             }),
+    ///             None).await?;
+    ///         Ok(value)
+    ///     })
+    /// }
+    /// ```
+    pub async fn read_rels(
+        &self,
+        src_node_label: &str,
+        rel_label: &str,
+        input: impl TryInto<Value>,
+        partition_key_opt: Option<&Value>,
+        transaction: &'a mut <<<RequestCtx as RequestContext>::DBEndpointType as DatabaseEndpoint>::PoolType as DatabasePool>::TransactionType,
+    ) -> Result<Vec<Rel<RequestCtx>>, Error> {
+        let input_value_opt = Some(input.try_into().map_err(|_e| Error::TypeConversionFailed {
+            src: "TryInto<Value>".to_string(),
+            dst: "Value".to_string(),
+        })?);
+        let mut sg = SuffixGenerator::new();
+        let rel_suffix = sg.suffix();
+        let dst_suffix = sg.suffix();
+        let src_var = NodeQueryVar::new(
+            Some(src_node_label.to_string()),
+            "src".to_string(),
+            sg.suffix(),
+        );
+        let dst_var = NodeQueryVar::new(None, "dst".to_string(), dst_suffix);
+        let rel_var = RelQueryVar::new(rel_label.to_string(), rel_suffix, src_var, dst_var);
+        let info = Info::new(
+            src_node_label.to_string()
+                + &*((&rel_label.to_string().to_title_case())
+                    .split_whitespace()
+                    .collect::<String>())
+                + "QueryInput",
+            self.info.type_defs(),
+        );
+        let query_fragment = visit_rel_query_input::<RequestCtx>(
+            None,
+            &rel_var,
+            input_value_opt,
+            &info,
+            partition_key_opt,
+            &mut sg,
+            transaction,
+        )
+        .await?;
+
+        let results = transaction
+            .read_rels(query_fragment, &rel_var, None, partition_key_opt)
+            .await?;
+
+        Ok(results)
+    }
+
     /// Provides an abstracted database node create operation using warpgrapher inputs. This is the
     /// recommended way to read data in a database-agnostic way that ensures the event handlers
     /// are portable across different databases.
@@ -399,22 +348,33 @@ where
     /// ```rust, no_run
     /// # use serde_json::json;
     /// # use warpgrapher::Error;
+    /// # use warpgrapher::engine::database::{DatabasePool, Transaction};
     /// # use warpgrapher::engine::resolvers::{ResolverFacade, ExecutionResult};
     /// # use warpgrapher::engine::value::Value;
     /// # use warpgrapher::juniper::BoxFuture;
     ///
     /// fn custom_resolve(mut facade: ResolverFacade<()>) -> BoxFuture<ExecutionResult> {
     ///     Box::pin(async move {
-    ///         let alice = facade.create_node("User", json!({"name": "alice"}), None).await?;
+    ///         let mut transaction = facade.executor().context().pool().transaction().await?;
+    ///         transaction.begin().await?;
+    ///         let result = ResolverFacade::create_node(&facade, "User", json!({"name": "alice"}), None, &mut transaction).await;
+    ///         if result.is_ok() {
+    ///             transaction.commit().await?;
+    ///         } else {
+    ///             transaction.rollback().await?;
+    ///         }
+    ///         let alice = result?;
+    ///         std::mem::drop(transaction);
     ///         facade.resolve_node(&alice).await
     ///     })
     /// }
     /// ```
     pub async fn create_node(
-        &mut self,
+        rf: &'a ResolverFacade<'_, RequestCtx>,
         type_name: &str,
         input: impl TryInto<Value>,
         partition_key_opt: Option<&Value>,
+        transaction: &'a mut <<<RequestCtx as RequestContext>::DBEndpointType as DatabaseEndpoint>::PoolType as DatabasePool>::TransactionType,
     ) -> Result<Node<RequestCtx>, Error> {
         let mut sg = SuffixGenerator::new();
         let node_var =
@@ -425,11 +385,11 @@ where
                 src: "".to_string(),
                 dst: "".to_string(),
             })?,
-            &Info::new(type_name.to_string(), self.info.type_defs()),
+            &Info::new(type_name.to_string(), rf.info.type_defs()),
             partition_key_opt,
             &mut sg,
-            self.transaction,
-            self.executor.context(),
+            transaction,
+            rf.executor.context(),
         )
         .await;
         result
@@ -450,13 +410,18 @@ where
     /// ```rust, no_run
     /// # use serde_json::json;
     /// # use warpgrapher::Error;
+    /// # use warpgrapher::engine::database::{DatabasePool, Transaction};
     /// # use warpgrapher::engine::resolvers::{ResolverFacade, ExecutionResult};
     /// # use warpgrapher::engine::value::Value;
     /// # use warpgrapher::juniper::BoxFuture;
     ///
     /// fn custom_resolve(mut facade: ResolverFacade<()>) -> BoxFuture<ExecutionResult> {
     ///     Box::pin(async move {
-    ///         let result = facade.update_node(
+    ///         let mut transaction = facade.executor().context().pool().transaction().await?;
+    ///         transaction.begin().await?;
+    ///
+    ///         let result = ResolverFacade::update_node(
+    ///             &facade,
     ///             "User",
     ///             json!({
     ///                 "MATCH": {
@@ -468,18 +433,24 @@ where
     ///                     "age": 20
     ///                 }
     ///             }),
-    ///             None
+    ///             None,
+    ///             &mut transaction
     ///         ).await?;
+    ///
+    ///         transaction.commit().await?;
+    ///         std::mem::drop(transaction);
+    ///
     ///         let alice = result.first().unwrap();
     ///         facade.resolve_node(&alice).await
     ///     })
     /// }
     /// ```
     pub async fn update_node(
-        &mut self,
+        rf: &ResolverFacade<'_, RequestCtx>,
         type_name: &str,
         input: impl TryInto<Value>,
         partition_key_opt: Option<&Value>,
+        transaction: &'a mut <<<RequestCtx as RequestContext>::DBEndpointType as DatabaseEndpoint>::PoolType as DatabasePool>::TransactionType,
     ) -> Result<Vec<Node<RequestCtx>>, Error> {
         let mut sg = SuffixGenerator::new();
         let node_var =
@@ -490,14 +461,11 @@ where
                 src: "".to_string(),
                 dst: "".to_string(),
             })?,
-            &Info::new(
-                format!("{}UpdateInput", type_name.to_string()),
-                self.info.type_defs(),
-            ),
+            &Info::new(format!("{}UpdateInput", type_name), rf.info.type_defs()),
             partition_key_opt,
             &mut sg,
-            self.transaction,
-            self.executor.context(),
+            transaction,
+            rf.executor.context(),
         )
         .await;
         result
@@ -518,13 +486,18 @@ where
     /// ```rust, no_run
     /// # use serde_json::json;
     /// # use warpgrapher::Error;
+    /// # use warpgrapher::engine::database::{DatabasePool, Transaction};
     /// # use warpgrapher::engine::resolvers::{ResolverFacade, ExecutionResult};
     /// # use warpgrapher::engine::value::Value;
     /// # use warpgrapher::juniper::BoxFuture;
     ///
     /// fn custom_resolve(mut facade: ResolverFacade<()>) -> BoxFuture<ExecutionResult> {
     ///     Box::pin(async move {
-    ///         facade.delete_node(
+    ///         let mut transaction = facade.executor().context().pool().transaction().await?;
+    ///         transaction.begin().await?;
+    ///
+    ///         ResolverFacade::delete_node(
+    ///             &facade,
     ///             "User",
     ///             json!({
     ///                 "MATCH": {
@@ -533,17 +506,23 @@ where
     ///                     }
     ///                 }
     ///             }),
-    ///             None
+    ///             None,
+    ///             &mut transaction
     ///         ).await?;
+    ///
+    ///         transaction.commit().await?;
+    ///         std::mem::drop(transaction);
+    ///
     ///         facade.resolve_null()
     ///     })
     /// }
     /// ```
     pub async fn delete_node(
-        &mut self,
+        rf: &ResolverFacade<'_, RequestCtx>,
         type_name: &str,
         input: impl TryInto<Value>,
         partition_key_opt: Option<&Value>,
+        transaction: &'a mut <<<RequestCtx as RequestContext>::DBEndpointType as DatabaseEndpoint>::PoolType as DatabasePool>::TransactionType,
     ) -> Result<Vec<Node<RequestCtx>>, Error> {
         let mut sg = SuffixGenerator::new();
         let node_var =
@@ -554,14 +533,11 @@ where
                 src: "".to_string(),
                 dst: "".to_string(),
             })?,
-            &Info::new(
-                format!("{}DeleteInput", type_name.to_string()),
-                self.info.type_defs(),
-            ),
+            &Info::new(format!("{}DeleteInput", type_name), rf.info.type_defs()),
             partition_key_opt,
             &mut sg,
-            self.transaction,
-            self.executor.context(),
+            transaction,
+            rf.executor.context(),
         )
         .await;
         result
@@ -621,7 +597,9 @@ where
                 },
             ))
         } else {
-            Err(Error::TypeNotExpected { details: None })
+            Err(Error::TypeNotExpected {
+                details: Some("parent_node is not of type Object::Node".to_string()),
+            })
         }
     }
 
@@ -678,7 +656,9 @@ where
                 NodeRef::Node(dst),
             ))
         } else {
-            Err(Error::TypeNotExpected { details: None })
+            Err(Error::TypeNotExpected {
+                details: Some("parent_node is not of type Object::Node".to_string()),
+            })
         }
     }
 
@@ -760,7 +740,9 @@ where
         if let Object::Node(n) = self.parent {
             Ok(n)
         } else {
-            Err(Error::TypeNotExpected { details: None })
+            Err(Error::TypeNotExpected {
+                details: Some("parent_node is not of type Object::Node".to_string()),
+            })
         }
     }
 
@@ -865,6 +847,41 @@ where
             .await
     }
 
+    /// Returns a GraphQL Object representing a list of graph node defined by a type and a map of props.
+    ///
+    /// # Examples
+    /// ```rust, no_run
+    /// use serde_json::json;
+    /// use std::collections::HashMap;
+    /// use warpgrapher::engine::resolvers::{ExecutionResult, ResolverFacade};
+    /// use warpgrapher::engine::value::Value;
+    /// use warpgrapher::juniper::BoxFuture;
+    ///
+    /// fn custom_resolve(facade: ResolverFacade<()>) -> BoxFuture<ExecutionResult> {
+    ///     Box::pin(async move {
+    ///         // do work
+    ///         let mut hm = HashMap::new();
+    ///         hm.insert("name".to_string(), Value::String("John Doe".to_string()));
+    ///         hm.insert("age".to_string(), Value::Int64(21));
+    ///
+    ///         // return node
+    ///         let node_list = vec![facade.node("User", hm)];
+    ///         facade.resolve_node_list(node_list).await
+    ///     })
+    /// }
+    /// ```
+    pub async fn resolve_node_list(&self, node_list: Vec<Node<RequestCtx>>) -> ExecutionResult {
+        self.executor
+            .resolve_async(
+                &Info::new(
+                    node_list.first().unwrap().typename().to_string(),
+                    self.info.type_defs(),
+                ),
+                &node_list,
+            )
+            .await
+    }
+
     /// Returns a GraphQL Object representing a graph relationship defined by an ID, props, and a
     /// destination Warpgrapher Node.
     ///
