@@ -1,40 +1,34 @@
-#[cfg(feature = "neo4j")]
-use bolt_proto::Message;
 #[cfg(feature = "gremlin")]
 use gremlin_client::TlsOptions;
-#[cfg(any(feature = "cosmos", feature = "gremlin"))]
+#[cfg(feature = "gremlin")]
 use gremlin_client::{ConnectionOptions, GraphSON, GremlinClient};
 #[cfg(any(feature = "neo4j"))]
 use log::trace;
 #[cfg(feature = "neo4j")]
-use mobc::Connection;
-#[cfg(feature = "neo4j")]
-use mobc_boltrs::BoltConnectionManager;
-#[cfg(feature = "neo4j")]
 use std::collections::HashMap;
 #[cfg(feature = "neo4j")]
 use std::convert::TryFrom;
-#[cfg(any(feature = "cosmos", feature = "gremlin", feature = "neo4j"))]
+#[cfg(any(feature = "gremlin", feature = "neo4j"))]
 use std::convert::TryInto;
-#[cfg(any(feature = "cosmos", feature = "gremlin"))]
+#[cfg(feature = "gremlin")]
 use std::env::var_os;
 use std::fs::File;
 use std::io::BufReader;
-#[cfg(feature = "neo4j")]
-use std::iter::FromIterator;
 use warpgrapher::engine::context::RequestContext;
 #[cfg(feature = "gremlin")]
 use warpgrapher::engine::database::env_bool;
-#[cfg(feature = "cosmos")]
-use warpgrapher::engine::database::gremlin::CosmosEndpoint;
 #[cfg(feature = "gremlin")]
-use warpgrapher::engine::database::gremlin::{GremlinEndpoint, NeptuneEndpoint};
+use warpgrapher::engine::database::gremlin::GremlinEndpoint;
 #[cfg(feature = "neo4j")]
 use warpgrapher::engine::database::neo4j::Neo4jEndpoint;
-#[cfg(any(feature = "cosmos", feature = "gremlin", feature = "neo4j"))]
+#[cfg(feature = "neo4j")]
+use warpgrapher::engine::database::neo4j::Neo4jTransaction;
+#[cfg(any(feature = "gremlin", feature = "neo4j"))]
 use warpgrapher::engine::database::DatabaseEndpoint;
 #[cfg(feature = "neo4j")]
-use warpgrapher::engine::database::{DatabaseClient, DatabasePool};
+use warpgrapher::engine::database::QueryResult;
+#[cfg(feature = "neo4j")]
+use warpgrapher::engine::database::{DatabasePool, Transaction};
 #[cfg(feature = "neo4j")]
 use warpgrapher::engine::events::EventHandlerBag;
 #[cfg(feature = "neo4j")]
@@ -48,7 +42,7 @@ use warpgrapher::engine::validators::Validators;
 use warpgrapher::engine::value::Value;
 #[cfg(feature = "neo4j")]
 use warpgrapher::juniper::BoxFuture;
-#[cfg(any(feature = "cosmos", feature = "gremlin", feature = "neo4j"))]
+#[cfg(any(feature = "gremlin", feature = "neo4j"))]
 use warpgrapher::{Client, Engine};
 use warpgrapher::{Configuration, Error};
 
@@ -57,45 +51,6 @@ pub(crate) fn init() {
     let _ = env_logger::builder().is_test(true).try_init();
 }
 
-#[cfg(feature = "cosmos")]
-fn cosmos_host() -> String {
-    var_os("WG_COSMOS_HOST")
-        .expect("Expected WG_COSMOS_HOST to be set.")
-        .to_str()
-        .expect("Expected WG_COSMOS_HOST to be a string.")
-        .to_owned()
-}
-
-#[cfg(feature = "cosmos")]
-fn cosmos_port() -> u16 {
-    var_os("WG_COSMOS_PORT")
-        .expect("Expected WG_COSMOS_PORT to be set.")
-        .to_str()
-        .expect("Expected WG_COSMOS_PORT to be a string.")
-        .parse::<u16>()
-        .expect("Expected WG_COSMOS_PORT to be a u16.")
-}
-
-#[allow(dead_code)]
-#[cfg(feature = "cosmos")]
-fn cosmos_user() -> String {
-    var_os("WG_COSMOS_USER")
-        .expect("Expected WG_COSMOS_USER to be set.")
-        .to_str()
-        .expect("Expected WG_COSMOS_USER to be a string.")
-        .to_owned()
-}
-
-#[allow(dead_code)]
-#[cfg(feature = "cosmos")]
-fn cosmos_pass() -> String {
-    var_os("WG_COSMOS_PASS")
-        .expect("Expected WG_COSMOS_PASS to be set.")
-        .to_str()
-        .expect("Expected WG_COSMOS_PASS to be a string.")
-        .to_owned()
-}
-
 #[cfg(feature = "gremlin")]
 fn gremlin_host() -> String {
     var_os("WG_GREMLIN_HOST")
@@ -136,20 +91,16 @@ fn gremlin_use_tls() -> bool {
 #[allow(dead_code)]
 #[cfg(feature = "gremlin")]
 fn gremlin_accept_invalid_tls() -> bool {
-    env_bool("WG_GREMLIN_CERT").unwrap_or(true)
+    env_bool("WG_GREMLIN_VALIDATE_CERTS").unwrap_or(true)
 }
 
 #[allow(dead_code)]
 #[cfg(feature = "neo4j")]
-pub(crate) async fn bolt_client() -> Result<Box<Connection<BoltConnectionManager>>, Error> {
+pub(crate) async fn bolt_transaction() -> Result<Neo4jTransaction, Error> {
     let endpoint = Neo4jEndpoint::from_env()?;
     let pool = endpoint.pool().await?;
 
-    if let DatabaseClient::Neo4j(c) = pool.client().await? {
-        Ok(c)
-    } else {
-        Err(Error::DatabaseNotFound)
-    }
+    pool.transaction().await
 }
 
 #[allow(dead_code)]
@@ -209,45 +160,6 @@ pub(crate) async fn neo4j_test_client_with_events(
     Client::new_with_engine(engine, None)
 }
 
-#[allow(dead_code)]
-#[cfg(feature = "cosmos")]
-pub(crate) async fn cosmos_test_client(config_path: &str) -> Client<CosmosRequestCtx> {
-    // load config
-    //let config_path = "./tests/fixtures/config.yml".to_string();
-    let config: Configuration = File::open(config_path)
-        .expect("Failed to load config file")
-        .try_into()
-        .unwrap();
-
-    let database_pool = CosmosEndpoint::from_env().unwrap().pool().await.unwrap();
-
-    let engine = Engine::<CosmosRequestCtx>::new(config, database_pool)
-        .with_version("1.0".to_string())
-        .build()
-        .expect("Could not create warpgrapher engine");
-
-    Client::new_with_engine(engine, None)
-}
-
-#[cfg(feature = "cosmos")]
-#[allow(dead_code)]
-fn clear_cosmos_db() {
-    // g.V().drop() -- delete the entire graph
-    let client = GremlinClient::connect(
-        ConnectionOptions::builder()
-            .host(cosmos_host())
-            .port(cosmos_port())
-            .pool_size(1)
-            .ssl(true)
-            .serializer(GraphSON::V2)
-            .credentials(&cosmos_user(), &cosmos_pass())
-            .build(),
-    )
-    .expect("Expected successful gremlin client creation.");
-
-    let _ = client.execute("g.V().drop()", &[]);
-}
-
 #[allow(dead_code)]
 #[cfg(feature = "gremlin")]
 pub(crate) async fn gremlin_test_client(config_path: &str) -> Client<GremlinRequestCtx> {
@@ -294,24 +206,19 @@ fn clear_gremlin_db() {
 #[cfg(feature = "neo4j")]
 #[allow(dead_code)]
 async fn clear_neo4j_db() {
-    let mut graph = bolt_client().await.expect("Failed to get database client");
-    let result = graph
-        .run_with_metadata("MATCH (n) DETACH DELETE (n);", None, None)
-        .await;
-    result.expect("Expected successful query run.");
-
-    let pull_meta = bolt_client::Metadata::from_iter(vec![("n", 1)]);
-    let (_response, _records) = graph
-        .pull(Some(pull_meta))
+    bolt_transaction()
         .await
-        .expect("Expected pull to succeed.");
+        .expect("Failed to get database client")
+        .execute_query::<Neo4jRequestCtx>(
+            "MATCH (n) DETACH DELETE (n);".to_string(),
+            HashMap::new(),
+        )
+        .await
+        .expect("Expected successful query run.");
 }
 
 #[allow(dead_code)]
 pub(crate) async fn clear_db() {
-    #[cfg(feature = "cosmos")]
-    clear_cosmos_db();
-
     #[cfg(feature = "gremlin")]
     clear_gremlin_db();
 
@@ -321,13 +228,16 @@ pub(crate) async fn clear_db() {
 
 #[derive(Clone, Debug)]
 pub struct Metadata {
+    #[allow(dead_code)]
     pub(crate) src_ip: String,
+    #[allow(dead_code)]
     pub(crate) src_useragent: String,
 }
 
 #[cfg(feature = "neo4j")]
 #[derive(Clone, Debug)]
 pub struct Neo4jRequestCtx {
+    #[allow(dead_code)]
     metadata: Metadata,
 }
 
@@ -344,46 +254,9 @@ impl RequestContext for Neo4jRequestCtx {
     }
 }
 
-/*
-#[cfg(feature = "neo4j")]
-impl MetadataExtensionCtx for Neo4jRequestCtx {
-    fn set_metadata(&mut self, metadata: Metadata) {
-        self.metadata = metadata
-    }
-}
-*/
-
-#[cfg(feature = "cosmos")]
-#[derive(Clone, Debug)]
-pub struct CosmosRequestCtx {
-    metadata: Metadata,
-}
-
-#[cfg(feature = "cosmos")]
-impl RequestContext for CosmosRequestCtx {
-    type DBEndpointType = CosmosEndpoint;
-    fn new() -> CosmosRequestCtx {
-        CosmosRequestCtx {
-            metadata: Metadata {
-                src_ip: "".to_string(),
-                src_useragent: "".to_string(),
-            },
-        }
-    }
-}
-
-/*
-#[cfg(feature = "cosmos")]
-impl MetadataExtensionCtx for CosmosRequestCtx {
-    fn set_metadata(&mut self, metadata: Metadata) {
-        self.metadata = metadata
-    }
-}
-*/
-
-#[cfg(feature = "gremlin")]
 #[derive(Clone, Debug)]
 pub struct GremlinRequestCtx {
+    #[allow(dead_code)]
     metadata: Metadata,
 }
 
@@ -400,33 +273,6 @@ impl RequestContext for GremlinRequestCtx {
     }
 }
 
-#[cfg(feature = "gremlin")]
-#[derive(Clone, Debug)]
-pub struct NeptuneRequestCtx {
-    metadata: Metadata,
-}
-
-#[cfg(feature = "gremlin")]
-impl RequestContext for NeptuneRequestCtx {
-    type DBEndpointType = NeptuneEndpoint;
-    fn new() -> NeptuneRequestCtx {
-        NeptuneRequestCtx {
-            metadata: Metadata {
-                src_ip: "".to_string(),
-                src_useragent: "".to_string(),
-            },
-        }
-    }
-}
-/*
-#[cfg(feature = "gremlin")]
-impl MetadataExtensionCtx for GremlinRequestCtx {
-    fn set_metadata(&mut self, metadata: Metadata) {
-        self.metadata = metadata
-    }
-}
-*/
-
 #[allow(dead_code)]
 fn name_validator(value: &Value) -> Result<(), Error> {
     let name = match value {
@@ -476,19 +322,19 @@ fn name_validator(value: &Value) -> Result<(), Error> {
 #[cfg(feature = "neo4j")]
 pub(crate) fn project_count(facade: ResolverFacade<Neo4jRequestCtx>) -> BoxFuture<ExecutionResult> {
     Box::pin(async move {
-        let mut db = facade.db_into_neo4j().await?;
-        let query = "MATCH (n:Project) RETURN (n);";
-        db.run_with_metadata(query, None, None)
-            .await
-            .expect("Expected successful query run.");
-        let pull_meta = bolt_client::Metadata::from_iter(vec![("n", -1)]);
-        let (response, records) = db.pull(Some(pull_meta)).await?;
-        match response {
-            Message::Success(_) => (),
-            message => return Err(Error::Neo4jQueryFailed { message }.into()),
+        let query = "MATCH (n:Project) RETURN (n);".to_string();
+        let mut transaction = facade.executor().context().pool().transaction().await?;
+        transaction.begin().await?;
+        if let QueryResult::Neo4j(records) = transaction
+            .execute_query::<Neo4jRequestCtx>(query, HashMap::new())
+            .await?
+        {
+            transaction.commit().await?;
+            std::mem::drop(transaction);
+            facade.resolve_scalar(records.len() as i32)
+        } else {
+            Err(warpgrapher::Error::TypeNotExpected { details: None }.into())
         }
-
-        facade.resolve_scalar(records.len() as i32)
     })
 }
 
@@ -537,42 +383,42 @@ pub(crate) fn project_top_dev(
     facade: ResolverFacade<Neo4jRequestCtx>,
 ) -> BoxFuture<ExecutionResult> {
     Box::pin(async move {
-        let mut db = facade.db_into_neo4j().await?;
-        let query = "MATCH (n:User) RETURN (n);";
-        db.run_with_metadata(query, None, None)
+        let mut transaction = facade.executor().context().pool().transaction().await?;
+        transaction.begin().await?;
+        let query = "MATCH (n:User) RETURN (n);".to_string();
+        let qr = transaction
+            .execute_query::<Neo4jRequestCtx>(query, HashMap::new())
             .await
             .expect("Expected successful query run.");
 
-        let pull_meta = bolt_client::Metadata::from_iter(vec![("n", -1)]);
-        let (response, records) = db.pull(Some(pull_meta)).await?;
-        match response {
-            Message::Success(_) => (),
-            message => return Err(Error::Neo4jQueryFailed { message }.into()),
-        }
+        if let QueryResult::Neo4j(result) = qr {
+            let dev_id = if let bolt_proto::value::Value::Node(n) =
+                &result.get(0).expect("Expected result").fields()[0]
+            {
+                Value::try_from(n.properties().get("id").expect("Expected id").clone())
+                    .expect("Expected string")
+            } else {
+                panic!("Expected node.")
+            };
 
-        let dev_id = if let bolt_proto::value::Value::Node(n) =
-            &records.get(0).expect("Expected result").fields()[0]
-        {
-            Value::try_from(n.properties().get("id").expect("Expected id").clone())
-                .expect("Expected string")
+            transaction.commit().await?;
+            std::mem::drop(transaction);
+
+            facade
+                .resolve_rel(
+                    &facade
+                        .create_rel(
+                            Value::String("1234567890".to_string()),
+                            None,
+                            dev_id,
+                            "User",
+                        )
+                        .expect("Expected new rel"),
+                )
+                .await
         } else {
-            panic!("Expected node.")
-        };
-
-        std::mem::drop(db);
-
-        facade
-            .resolve_rel(
-                &facade
-                    .create_rel(
-                        Value::String("1234567890".to_string()),
-                        None,
-                        dev_id,
-                        "User",
-                    )
-                    .expect("Expected new rel"),
-            )
-            .await
+            Err(warpgrapher::Error::TypeNotExpected { details: None }.into())
+        }
     })
 }
 
@@ -582,50 +428,51 @@ pub(crate) fn project_top_issues(
     facade: ResolverFacade<Neo4jRequestCtx>,
 ) -> BoxFuture<ExecutionResult> {
     Box::pin(async move {
-        let mut db = facade.db_into_neo4j().await?;
-        let query = "MATCH (n:Bug) RETURN (n);";
-        db.run_with_metadata(query, None, None)
-            .await
-            .expect("Expected successful query run.");
+        let mut transaction = facade.executor().context().pool().transaction().await?;
+        transaction.begin().await?;
+        let query = "MATCH (n:Bug) RETURN (n);".to_string();
 
-        let pull_meta = bolt_client::Metadata::from_iter(vec![("n", -1)]);
-        let (response, records) = db.pull(Some(pull_meta)).await?;
-        match response {
-            Message::Success(_) => (),
-            message => return Err(Error::Neo4jQueryFailed { message }.into()),
-        }
+        let qr = transaction
+            .execute_query::<Neo4jRequestCtx>(query, HashMap::new())
+            .await?;
 
-        let bug_id = if let bolt_proto::value::Value::Node(n) =
-            &records.get(0).expect("Expected result").fields()[0]
-        {
-            Value::try_from(n.properties().get("id").expect("Expected id").clone())
-                .expect("Expected string")
+        let bug_id = if let QueryResult::Neo4j(records) = qr {
+            if let bolt_proto::value::Value::Node(n) =
+                &records.get(0).expect("Expected result").fields()[0]
+            {
+                Value::try_from(n.properties().get("id").expect("Expected id").clone())
+                    .expect("Expected string")
+            } else {
+                transaction.rollback().await?;
+                panic!("Expected node.")
+            }
         } else {
-            panic!("Expected node.")
+            transaction.rollback().await?;
+            panic!("Expected Neo4j records");
         };
 
-        let query = "MATCH (n:Feature) RETURN (n);";
-        db.run_with_metadata(query, None, None)
-            .await
-            .expect("Expected successful query run.");
+        let query = "MATCH (n:Feature) RETURN (n);".to_string();
+        let qr = transaction
+            .execute_query::<Neo4jRequestCtx>(query, HashMap::new())
+            .await?;
 
-        let pull_meta = bolt_client::Metadata::from_iter(vec![("n", -1)]);
-        let (response, records) = db.pull(Some(pull_meta)).await?;
-        match response {
-            Message::Success(_) => (),
-            message => return Err(Error::Neo4jQueryFailed { message }.into()),
-        }
-
-        let feature_id = if let bolt_proto::value::Value::Node(n) =
-            &records.get(0).expect("Expected result").fields()[0]
-        {
-            Value::try_from(n.properties().get("id").expect("Expected id").clone())
-                .expect("Expected string")
+        let feature_id = if let QueryResult::Neo4j(records) = qr {
+            if let bolt_proto::value::Value::Node(n) =
+                &records.get(0).expect("Expected result").fields()[0]
+            {
+                Value::try_from(n.properties().get("id").expect("Expected id").clone())
+                    .expect("Expected string")
+            } else {
+                transaction.rollback().await?;
+                panic!("Expected node.")
+            }
         } else {
-            panic!("Expected node.")
+            transaction.rollback().await?;
+            panic!("Expected Neo4j records");
         };
 
-        std::mem::drop(db);
+        transaction.commit().await?;
+        std::mem::drop(transaction);
 
         facade
             .resolve_rel_list(vec![
